@@ -2,7 +2,7 @@
  * ----------------------------------------------------------------------------
  * Imlib2 wrapper for Python
  * ----------------------------------------------------------------------------
- * $Id: image.c 2604 2007-03-28 09:21:52Z dmeyer $
+ * $Id: image.c 2941 2008-01-05 19:51:25Z tack $
  *
  * ----------------------------------------------------------------------------
  * kaa.imlib2 - An imlib2 wrapper for Python
@@ -105,35 +105,50 @@ Imlib_Image *imlib_image_from_pyobject(Image_PyObject *pyimg)
 
 Py_ssize_t Image_PyObject_Buffer__get_read_buffer(PyObject *self, Py_ssize_t segment, void **ptr)
 {
+    Py_ssize_t size;
+
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     if (ptr)
         *ptr = (void *)imlib_image_get_data_for_reading_only();
-    return imlib_image_get_width() * imlib_image_get_height() * 4;
+    size = imlib_image_get_width() * imlib_image_get_height() * 4;
+    PyImlib2_END_CRITICAL_SECTION
+
+    return size;
 }
 
 Py_ssize_t Image_PyObject_Buffer__get_readwrite_buffer(PyObject *self, Py_ssize_t segment, void **ptr)
 {
     Image_PyObject *o = (Image_PyObject *)self;
-    imlib_context_set_image(o->image);
+    Py_ssize_t size;
+
     if (segment > 0) {
         PyErr_Format(PyExc_SystemError, "Invalid segment for read/write buffer.");
         return -1;
     }
+
+    PyImlib2_BEGIN_CRITICAL_SECTION
+    imlib_context_set_image(o->image);
     if (ptr) {
         if (o->raw_data)
             *ptr = o->raw_data;
         else
             *ptr = o->raw_data = (void *)imlib_image_get_data();
     }
-    return imlib_image_get_width() * imlib_image_get_height() * 4;
+    size = imlib_image_get_width() * imlib_image_get_height() * 4;
+    PyImlib2_END_CRITICAL_SECTION
+
+    return size;
 }
 
 Py_ssize_t Image_PyObject_Buffer__get_seg_count(PyObject *self, Py_ssize_t *lenp)
 {
+    PyImlib2_BEGIN_CRITICAL_SECTION
     if (lenp) {
         imlib_context_set_image(((Image_PyObject *)self)->image);
         *lenp = (Py_ssize_t)(imlib_image_get_width() * imlib_image_get_height() * 4);
     }
+    PyImlib2_END_CRITICAL_SECTION
     return 1;
 }
 
@@ -148,8 +163,11 @@ Image_PyObject *_new_image_pyobject(Imlib_Image *image)
 
 void Image_PyObject__dealloc(Image_PyObject *self)
 {
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(self->image);
     imlib_free_image();
+    PyImlib2_END_CRITICAL_SECTION
+
     if (self->buffer) {
         Py_DECREF(self->buffer);
     }
@@ -166,6 +184,7 @@ PyObject *Image_PyObject__clear(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "iiii", &x, &y, &w, &h))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     data = (unsigned char *)imlib_image_get_data();
     max_w = imlib_image_get_width();
@@ -179,6 +198,7 @@ PyObject *Image_PyObject__clear(PyObject *self, PyObject *args)
     for (cur_y = y; cur_y < y + h; cur_y++)
         memset(&data[cur_y*max_w*4+(x*4)], 0, 4*w);
     imlib_image_put_back_data((DATA32 *)data);
+    PyImlib2_END_CRITICAL_SECTION
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -195,9 +215,12 @@ PyObject *Image_PyObject__scale(PyObject *self, PyObject *args)
               &dst_h))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     image = imlib_create_cropped_scaled_image(x, y, src_w, src_h,
                           dst_w, dst_h);
+    PyImlib2_END_CRITICAL_SECTION
+
     if (!image) {
         PyErr_Format(PyExc_RuntimeError, "Failed scaling image (%d, %d)",
              dst_w, dst_h);
@@ -218,9 +241,11 @@ PyObject *Image_PyObject__rotate(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "d", &angle))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
-
     image = imlib_create_rotated_image(angle);
+    PyImlib2_END_CRITICAL_SECTION
+
     if (!image) {
         PyErr_Format(PyExc_RuntimeError, "Failed rotating image (%f) degrees",
              angle);
@@ -239,8 +264,11 @@ PyObject *Image_PyObject__orientate(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "i", &orientation))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_image_orientate(orientation);
+    PyImlib2_END_CRITICAL_SECTION
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -252,10 +280,12 @@ PyObject *Image_PyObject__flip(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "iii", &horiz, &vert, &diag))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     if (horiz) imlib_image_flip_horizontal();
     if (vert)  imlib_image_flip_vertical();
     if (diag)  imlib_image_flip_diagonal();
+    PyImlib2_END_CRITICAL_SECTION
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -269,8 +299,11 @@ PyObject *Image_PyObject__blur(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "i", &radius))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_image_blur(radius);
+    PyImlib2_END_CRITICAL_SECTION
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -283,8 +316,11 @@ PyObject *Image_PyObject__sharpen(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "i", &radius))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_image_sharpen(radius);
+    PyImlib2_END_CRITICAL_SECTION
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -295,8 +331,11 @@ PyObject *Image_PyObject__clone(PyObject *self, PyObject *args)
     Imlib_Image *image;
     Image_PyObject *o;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     image = imlib_clone_image();
+    PyImlib2_END_CRITICAL_SECTION
+
     if (!image) {
         PyErr_Format(PyExc_RuntimeError, "Failed to clone image");
     return NULL;
@@ -326,6 +365,7 @@ PyObject *Image_PyObject__blend(PyObject *self, PyObject *args)
         return Py_None;
     }
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     src_img = ((Image_PyObject *)src)->image;
 
     if (src_alpha < 255) {
@@ -349,6 +389,7 @@ PyObject *Image_PyObject__blend(PyObject *self, PyObject *args)
                      dst_x, dst_y, dst_w, dst_h);
     imlib_context_set_blend(1);
     imlib_context_set_color_modifier(NULL);
+    PyImlib2_END_CRITICAL_SECTION
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -368,6 +409,7 @@ PyObject *Image_PyObject__draw_mask(PyObject *self, PyObject *args)
               &dst_y))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)mask)->image);
     mask_w = imlib_image_get_width();
     mask_h = imlib_image_get_height();
@@ -395,19 +437,12 @@ PyObject *Image_PyObject__draw_mask(PyObject *self, PyObject *args)
 
             // Blend average (grayscale) pixel from the mask with the
             // alpha channel of the image
-#if 1
-            // This is the code from Tack, but it doesn't work
-            // like it should ... -- Dischi
-            // (tack) Well, should work like this:
             int temp = (dst_chunk[3] * avg) + 0x80;
             dst_chunk[3] = ((temp + (temp >> 8)) >> 8);
-#else
-            // ... so this is my guess -- Dischi
-            dst_chunk[3] = (dst_chunk[3] * avg) / 255;
-#endif
         }
     }
     imlib_image_put_back_data((DATA32 *)dst_data);
+    PyImlib2_END_CRITICAL_SECTION
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -424,12 +459,14 @@ PyObject *Image_PyObject__draw_text(PyObject *self, PyObject *args)
 			  &y, &text, &r, &g, &b, &a))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_context_set_font(((Font_PyObject *)font)->font);
 
     imlib_context_set_color(r, g, b, a);
     imlib_text_draw_with_return_metrics(x, y, text, &w, &h, &advance_w,
 					&advance_h);
+    PyImlib2_END_CRITICAL_SECTION
     return Py_BuildValue("(llll)", w, h, advance_w, advance_h);
 }
 
@@ -464,6 +501,7 @@ PyObject *Image_PyObject__draw_text_with_style(PyObject *self, PyObject *args)
 			  &glow2.r, &glow2.g, &glow2.b, &glow2.a))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_context_set_font(((Font_PyObject *)font)->font);
 
@@ -546,6 +584,7 @@ PyObject *Image_PyObject__draw_text_with_style(PyObject *self, PyObject *args)
 
     COLOR_SET(color);
     imlib_text_draw_with_return_metrics(x, y, text, &w, &h, &advance_w, &advance_h);
+    PyImlib2_END_CRITICAL_SECTION
 
     /* FIXME: add effect to advance_w and advance_h */
     return Py_BuildValue("(llll)", w, h, advance_w, advance_h);
@@ -561,6 +600,7 @@ PyObject *Image_PyObject__draw_rectangle(PyObject *self, PyObject *args)
               &fill))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_image_set_has_alpha(1);
     imlib_context_set_color(r, g, b, a);
@@ -568,6 +608,7 @@ PyObject *Image_PyObject__draw_rectangle(PyObject *self, PyObject *args)
         imlib_image_draw_rectangle(x, y, w, h);
     else
         imlib_image_fill_rectangle(x, y, w, h);
+    PyImlib2_END_CRITICAL_SECTION
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -582,6 +623,7 @@ PyObject *Image_PyObject__draw_ellipse(PyObject *self, PyObject *args)
               &a, &fill))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_context_set_color(r, g, b, a);
     imlib_context_set_anti_alias(1);
@@ -589,6 +631,7 @@ PyObject *Image_PyObject__draw_ellipse(PyObject *self, PyObject *args)
         imlib_image_draw_ellipse(xc, yc, ea, eb);
     else
         imlib_image_fill_ellipse(xc, yc, ea, eb);
+    PyImlib2_END_CRITICAL_SECTION
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -601,8 +644,12 @@ PyObject *Image_PyObject__set_alpha(PyObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "i", &alpha))
         return NULL;
+
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_image_set_has_alpha(alpha);
+    PyImlib2_END_CRITICAL_SECTION
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -615,8 +662,11 @@ PyObject *Image_PyObject__copy_rect(PyObject *self, PyObject *args)
               &dst_y))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_image_copy_rect(src_x, src_y, w, h, dst_x, dst_y);
+    PyImlib2_END_CRITICAL_SECTION
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -629,8 +679,10 @@ PyObject *Image_PyObject__get_pixel(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "(ii)", &x, &y))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     imlib_image_query_pixel(x, y, &col);
+    PyImlib2_END_CRITICAL_SECTION
 
     return Py_BuildValue("(iiii)", col.blue, col.green, col.red, col.alpha);
 }
@@ -652,7 +704,6 @@ PyObject *Image_PyObject__get_raw_data(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "si", &format, &write))
         return NULL;
 
-    imlib_context_set_image(o->image);
     if (!strcmp(format, "BGRA")) {
         // Requested native format, so create a buffer directly from the
         // Image pyobject.
@@ -664,10 +715,20 @@ PyObject *Image_PyObject__get_raw_data(PyObject *self, PyObject *args)
         // Requested different format, create a new buffer.
         PyObject *buffer;
         unsigned char *data;
+        unsigned int size;
 
-        buffer = PyBuffer_New(get_raw_bytes_size(format));
+        PyImlib2_BEGIN_CRITICAL_SECTION
+        imlib_context_set_image(o->image);
+        size = get_raw_bytes_size(format);
+        PyImlib2_END_CRITICAL_SECTION
+
+        buffer = PyBuffer_New(size);
         PyObject_AsWriteBuffer(buffer, (void **)&data, &len);
+
+        PyImlib2_BEGIN_CRITICAL_SECTION
         get_raw_bytes(format, data);
+        PyImlib2_END_CRITICAL_SECTION
+
         return buffer;
     }
 }
@@ -683,13 +744,17 @@ PyObject *Image_PyObject__put_back_raw_data(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "O!", &PyBuffer_Type, &buffer_object))
         return NULL;
 
-    imlib_context_set_image(o->image);
     PyObject_AsWriteBuffer(buffer_object, (void **)&buffer, &len);
     if (buffer != o->raw_data) {
         PyErr_Format(PyExc_ValueError, "Putting back a buffer that wasn't gotten with get_raw_data()!");
         return NULL;
     }
+
+    PyImlib2_BEGIN_CRITICAL_SECTION
+    imlib_context_set_image(o->image);
     imlib_image_put_back_data((DATA32 *)buffer);
+    PyImlib2_END_CRITICAL_SECTION
+
     o->raw_data = NULL;
     Py_INCREF(Py_None);
     return Py_None;
@@ -703,6 +768,7 @@ PyObject *Image_PyObject__save(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "ss", &filename, &ext))
         return NULL;
 
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(((Image_PyObject *)self)->image);
     // TODO: call imlib_save_image_with_error_return
 
@@ -710,6 +776,8 @@ PyObject *Image_PyObject__save(PyObject *self, PyObject *args)
     /* argument - i.e. .png = png, .tif = tiff etc. */
     imlib_image_set_format(ext);
     imlib_save_image(filename);
+    PyImlib2_END_CRITICAL_SECTION
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -741,21 +809,36 @@ PyMethodDef Image_PyObject_methods[] = {
 
 PyObject *Image_PyObject__getattr(Image_PyObject *self, char *name)
 {
+    void *value = NULL;
+    int found = 1;
+    char *value_type = "i";
+
+    PyImlib2_BEGIN_CRITICAL_SECTION
     imlib_context_set_image(self->image);
     if (!strcmp(name, "width"))
-        return Py_BuildValue("i", imlib_image_get_width());
+        value = (void *)imlib_image_get_width();
     else if (!strcmp(name, "height"))
-        return Py_BuildValue("i", imlib_image_get_height());
+        value = (void *)imlib_image_get_height();
     else if (!strcmp(name, "has_alpha"))
-        return Py_BuildValue("i", imlib_image_has_alpha());
-    else if (!strcmp(name, "rowstride"))
-        return Py_BuildValue("l", imlib_image_get_width() * 4);
-    else if (!strcmp(name, "format"))
-        return Py_BuildValue("s", imlib_image_format());
-    else if (!strcmp(name, "mode"))
-        return Py_BuildValue("s", "BGRA");
-    else if (!strcmp(name, "filename"))
-        return Py_BuildValue("s", imlib_image_get_filename());
+        value = (void *)((int)imlib_image_has_alpha());
+    else if (!strcmp(name, "rowstride")) {
+        value = (void *)(imlib_image_get_width() * 4);
+        value_type = "l";
+    } else if (!strcmp(name, "format")) {
+        value = (void *)imlib_image_format();
+        value_type = "s";
+    } else if (!strcmp(name, "mode")) {
+        value = (void *)"BGRA";
+        value_type = "s";
+    } else if (!strcmp(name, "filename")) {
+        value = (void *)imlib_image_get_filename();
+        value_type = "s";
+    } else
+        found = 0;
+    PyImlib2_END_CRITICAL_SECTION
+
+    if (found)
+        return Py_BuildValue(value_type, value);
 
     return Py_FindMethod(Image_PyObject_methods, (PyObject *)self, name);
 }
