@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------------
 # image.py - An Imlib2 image class
 # -----------------------------------------------------------------------------
-# $Id: image.py 2618 2007-04-13 14:38:44Z tack $
+# $Id: image.py 2959 2008-01-08 16:53:03Z dmeyer $
 # -----------------------------------------------------------------------------
 # kaa.imlib2 - An imlib2 wrapper for Python
 # Copyright (C) 2004-2006 Dirk Meyer, Jason Tackaberry
@@ -35,8 +35,8 @@ import os
 
 # imlib2 wrapper
 import _Imlib2
+from kaa import Signal
 from kaa.strutils import utf8
-from kaa.notifier import Signal
 from font import *
 
 
@@ -425,10 +425,10 @@ class Image(object):
                  'font_or_fontname' is already a Font instance, it is simply
                  returned back to the caller.
         """
-        if type(font) in types.StringTypes:
-            self.font = Font(font)
+        if type(font_or_font_name) in types.StringTypes:
+            self.font = Font(font_or_font_name)
         else:
-            self.font = font
+            self.font = font_or_font_name
         return self.font
 
 
@@ -597,3 +597,15 @@ class Image(object):
         if not format:
             format = os.path.splitext(filename)[1][1:]
         return self._image.save(filename, format)
+
+
+    def as_gdk_pixbuf(self):
+        """
+        Returns the current image as a gdk.Pixbuf if pygtk is available.
+        Otherwise raises ImportError.
+        """
+        import gtk
+        data = self.get_raw_data('RGBA')
+        return gtk.gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, True, 8, 
+                                            self.width, self.height, self.width * 4)
+
